<?php

class DHWC_Layout_Admin{
	public function __construct(){
		add_action('admin_init', array($this, 'mce_init'));
		
		//Assets
		add_action( 'admin_print_scripts-post.php', array($this,'print_scripts') );
		add_action( 'admin_print_scripts-post-new.php', array($this,'print_scripts') );
		
		add_action('vc_backend_editor_enqueue_js_css', array($this,'print_scripts'));
		add_action('vc_frontend_editor_enqueue_js_css', array($this,'print_scripts'));
		
		//WC settings
		add_filter( 'woocommerce_get_settings_pages', array($this,'add_wc_settings') );
		
		//product category form
		add_action( 'product_cat_add_form_fields', array( $this, 'add_category_fields' ) );
		add_action( 'product_cat_edit_form_fields', array( $this, 'edit_category_fields' ) ,10,2);
		add_action( 'created_term', array( $this, 'save_category_fields' ), 10,3 );
		add_action( 'edit_term', array( $this, 'save_category_fields' ), 10,3);
		
		//product tag form 
		add_action( 'product_tag_add_form_fields', array( $this, 'add_tag_fields' ) );
		add_action( 'product_tag_edit_form_fields', array( $this, 'edit_tag_fields' ) ,10,2);
		add_action( 'created_term', array( $this, 'save_tag_fields' ) , 10,3);
		add_action( 'edit_term', array( $this, 'save_tag_fields' ), 10,3);
		
		//product brand form
		add_action( 'product_brand_add_form_fields', array( $this, 'add_brand_fields' ) );
		add_action( 'product_brand_edit_form_fields', array( $this, 'edit_brand_fields' ) ,10,2);
		add_action( 'created_term', array( $this, 'save_brand_fields' ), 10,3);
		add_action( 'edit_term', array( $this, 'save_brand_fields' ), 10,3);
		
	}
	
	public function print_scripts(){
		wp_enqueue_style('dhwc-layout-select2', DHVC_WOO_URL.'/assets/css/select2.min.css');
	}
	
	public function add_category_fields(){
		$this->_add_term_field(dhwc_layout_get_taxonomy_setting_field_id('product_cat'));
	}
	
	public function edit_category_fields( $term, $taxonomy ) {
		$this->_edit_term_field(dhwc_layout_get_taxonomy_setting_field_id('product_cat'), $term);
	}
	
	public function save_category_fields( $term_id, $tt_id, $taxonomy ) {
		$this->_save_term_field(dhwc_layout_get_taxonomy_setting_field_id('product_cat'), $term_id);
	}
	
	public function add_brand_fields(){
		$this->_add_term_field(dhwc_layout_get_taxonomy_setting_field_id('product_brand'));
	}
	
	public function edit_brand_fields( $term, $taxonomy ) {
		$this->_edit_term_field(dhwc_layout_get_taxonomy_setting_field_id('product_brand'), $term);
	}
	
	public function save_brand_fields( $term_id, $tt_id, $taxonomy ) {
		$this->_save_term_field(dhwc_layout_get_taxonomy_setting_field_id('product_brand'), $term_id);
	}
	
	public function add_tag_fields(){
		$this->_add_term_field(dhwc_layout_get_taxonomy_setting_field_id('product_tag'));
	}
	
	public function edit_tag_fields( $term, $taxonomy ) {
		$this->_edit_term_field(dhwc_layout_get_taxonomy_setting_field_id('product_tag'), $term);
	}
	
	public function save_tag_fields( $term_id, $tt_id, $taxonomy ) {
		$this->_save_term_field(dhwc_layout_get_taxonomy_setting_field_id('product_tag'), $term_id);
	}

	protected function _add_term_field($field_id){
		?>
		<div class="form-field">
			<label for="<?php echo esc_attr($field_id)?>"><?php _e( 'Content Page Template', DHVC_WOO ); ?></label>
			<?php 
			$args = array(
					'name'=>$field_id,
					'show_option_none'=>' ',
					'echo'=>false,
			);
			echo str_replace(' id=', " style='width:100%' data-placeholder='" . __( 'Select a page&hellip;',DHVC_WOO) .  "' class='wc-enhanced-select-nostd' id=", wp_dropdown_pages( $args ) );		
			?>
		</div>
		<?php 
	}
	
	protected function _edit_term_field($field_id, $term){
		$page_templage_id = get_term_meta( $term->term_id, $field_id, true );
		?>
		<tr class="form-field">
			<th scope="row" valign="top"><label><?php _e( 'Content Page Template', DHVC_WOO ); ?></label></th>
			<td>
				<?php 
				$args = array(
						'name'=>$field_id,
						'show_option_none'=>' ',
						'post_status' => 'publish,private',
						'echo'=>false,
						'selected'=>$page_templage_id
				);
				echo str_replace(' id=', " style='width:100%' data-placeholder='" . __( 'Select a page&hellip;',DHVC_WOO) .  "' class='wc-enhanced-select-nostd' id=", wp_dropdown_pages( $args ) );		
				?>
			</td>
		</tr>
		<?php
	}
	
	protected function _save_term_field( $field_id, $term_id ) {
		if(empty($field_id)){
			return;
		}
		if(!empty($_POST[$field_id])){
			update_term_meta($term_id, $field_id, $_POST[$field_id] );
		}else{
			delete_term_meta($term_id, $field_id);
		}
	}
	
	public function mce_init(){
		
		if (!current_user_can('edit_posts') && !current_user_can('edit_pages'))
			return;
		
		if ('true' === get_user_option('rich_editing')) {
			add_filter('mce_external_plugins', array($this, 'mce_external_plugins'));
			add_filter('mce_buttons', array($this, 'mce_buttons'));
		}
		
		
		wp_enqueue_script('wp-color-picker');
		wp_enqueue_style( 'wp-color-picker' );
		
		wp_enqueue_script('dhwc-layout-modal', DHVC_WOO_URL.'/assets/js/dhmodal.js',array('jquery'),DHVC_WOO_VERSION,true);
		
		$dhwc_layout = array(
			'plugin_url'						=> DHVC_WOO_URL,
			'shortcode_form_modal'				=> $this->_get_shortcode_form_modal(),
			'select_placeholder_text_single' 	=>	__('Select an option',DHVC_WOO)
		);
		wp_localize_script('dhwc-layout-modal', 'dhwc_layout_admin_params', $dhwc_layout);
		
		wp_enqueue_style('dhwc-layout-modal', DHVC_WOO_URL.'/assets/css/dhmodal.css');
		
		wp_enqueue_style('dhwc-layout-admin',DHVC_WOO_URL.'/assets/css/admin.css');
	}
	
	private function _get_shortcode_form_modal(){
		ob_start();
		?>
		<div class="dhmodal dhmodal--fade dhwc-layout-modal" tabindex="-1" role="dialog">
			<div class="dhmodal__dialog">
				<div class="dhmodal__content">
					<div class="dhmodal__header">
						<h3><?php esc_html_e('DHWC Layout Shortcode')?></h3>
						<a class="dhmodal__close" data-dismiss="dhmodal" href="#">&times;</a>
					</div>
					<div class="dhmodal__body">
						<?php echo $this->_get_shortcode_form()?>
					</div>
				</div>
			</div>
		</div>
		<?php
		return ob_get_clean();
	}
	
	private function _get_shortcode_form(){
		$params = dhwc_layout_get_setting_params();
		$params_fields = array (
			'products' 					=> 'dhwc_layout_param_field_products_ajax',
			'exclude_products'			=> 'dhwc_layout_param_field_products_ajax',
			'category' 					=> 'dhwc_layout_param_field_categories',
			'exclude_category' 			=> 'dhwc_layout_param_field_categories',
			'tag' 						=> 'dhwc_layout_param_field_tags',
			'exclude_tag' 				=> 'dhwc_layout_param_field_tags',
			'brand' 					=> 'dhwc_layout_param_field_brands',
			'exclude_brand'				=> 'dhwc_layout_param_field_brands',
			'attribute' 				=> 'dhwc_layout_param_field_attributes',
		);
		ob_start();
		?>
		<div class="dhwc-layout-shortcode-params">
		<?php
		foreach ($params as $param){
		?>
		<div class="dhwc-layout-shortcode-param">
			<?php if (isset($param['heading'])){?>
			<div class="dhwc-layout-element-label"><?php echo $param['heading'] ?></div>
			<?php } ?>
			<div class="dhwc-layout-element-input">
				<?php 
				$dependency = '';
				if(isset($param['dependency'])){
					$dependency_value = isset($param['dependency']['value']) ? $param['dependency']['value'] : '';
					if(is_array($dependency_value)){
						$dependency_value = implode(',', $dependency_value);	
					}
					$dependency = 'data-dependency="toggle" data-dependency-element="'.$param['dependency']['element'].'" data-dependency-value = "'.$dependency_value.'"';
				}
				
				if(array_key_exists($param['param_name'], $params_fields)){
					$func = $params_fields[$param['param_name']];
					$value = '';
					if(isset($param['value'])){
						$value = $param['value'];
					}
					echo call_user_func( $func, $param, $value, $dependency );
				}else{
					switch ($param['type']){
						case 'dhwc_layout_field_id':
							$value = '';
							if(isset($param['value'])){
								$value = $param['value'];
							}
							echo dhwc_layout_param_field_id($param, $value, $dependency);
						break;
						case 'dhwc_layout_field_heading':
							$value = '';
							if(isset($param['value'])){
								$value = $param['value'];
							}
							echo dhwc_layout_param_field_heading($param, $value, $dependency);
						break;
						case 'textfield':
							echo '<input '.$dependency.' id="'.$param['param_name'].'" class="dhwc-layout-param-value" name="'.$param['param_name'].'" value="'.(isset($param['value']) ? $param['value'] : '').'" type="text"/>';
							break;
						case 'dropdown':
							$param_line = '<select '.$dependency.' name="'.$param['param_name'].'" class="dhwc-layout-param-value" id="'.$param['param_name'].'">';	
							$param_value = isset($param['std']) ? $param['std'] : false;
							foreach ( $param['value'] as $text_val => $val ) {
								if ( is_numeric($text_val) && (is_string($val) || is_numeric($val)) ) {
									$text_val = $val;
								}
								$selected = '';
								if(false===$param_value){
									$param_value = $val;
								}
								if ((string)$val === (string)$param_value) {
									$selected = ' selected="selected"';
								}
								
								$param_line .= '<option value="'.$val.'"'.$selected.'>'.esc_html($text_val).'</option>';
							}
							$param_line .= '</select>';
							echo $param_line;
							break;
						case 'checkbox' :
							$values = is_array($param['value']) ? $param['value'] : array();
							$param_line = '';
							foreach ( $values as $label => $v ) {
								$checked ='';
								$param_line .= ' <input '.$dependency.' id="'. $param['param_name'] . '-' . $v .'" value="' . $v . '" class="dhwc-layout-param-value '.$param['param_name'].' '.$param['type'].'" type="checkbox" name="'.$param['param_name'].'"'.$checked.'> ' . __($label, DHVC_WOO);
							}
							echo $param_line;
							break;
						case 'colorpicker':
							echo '<div class="color-group">'
									.'<input '.$dependency.' name="'.$param['param_name'].'" class="dhwc-layout-param-value '.$param['type'].'_field dhwc-layout-color-control" value="'.$param['value'].'" type="text"/>'
								 .'</div>';
							break;
						default:
							break;
					}
				}
				?>
				<?php if(isset($param['description'])):?>
				<span class="clear" style="font-size: 11px;color:#999999"><?php echo $param['description'] ?></span>
				<?php endif;?>
			</div>
		</div>
		<?php
		}
		?>
		<p class="submit">
			<input id="dhwc-layout-shortcode-submit" class="button-primary" type="button" value="Insert Shortcode">
		</p>
		</div>
		<?php
		return ob_get_clean();
	}
	
	public function mce_external_plugins(){
		$plugins['dhwc_layout'] = DHVC_WOO_URL. '/assets/js/admin_plugin.js';
		return $plugins;
	}
	
	public function mce_buttons($buttons){
		$buttons[] = 'dhwc_layout_button';
		return $buttons;
	}
	
	public function add_wc_settings($settings){
		$settings[] = include DHVC_WOO_DIR.'/includes/class-dhwc-layout-admin-settings.php';
		return $settings;
	}
}
new DHWC_Layout_Admin();